// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class multiplying all negative values of an {@link
 * ALab.AObject} by <code>-1</code> and a factor abs. <br><br>
 * <IMG SRC="AAbs.gif" ALT="Diagram of the factor effect.">
 * <br>This factor can be modulated. If the multiplication is
 * out of range (<code>-1..1</code>) an exception is thrown.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 */
public class AAbs extends AObject {
  private double  abs;
  private AObject absMod = null;
  private AObject obj;

  /** Creates a new value absoluting instance.
   * @param o the object to be absoluted.
   * @param abs the factor to be multiplied. */
  public AAbs(AObject o, double abs) {
    super(o); setObj(o); setAbs(abs); }

  /** Creates a new <code>AAbs</code>
   * by copying the given <code>AAbs</code>.
   * @param o the given object. */
  protected AAbs(AAbs o) { super(o); setObj(o.getObj());
    setAbs(o.getAbs()); setAbsMod(o.getAbsMod()); }

  /** Returns the factor to be multiplied. */
  public double getAbs() { return abs; }

  /** Sets the factor to be multiplied.
   * @param v the factor value. */
  public void setAbs(double v) { abs = v; }

  /** Returns the factor modulator. */
  public AObject getAbsMod() { return absMod; }

  /** Sets the factor modulator to a copy of <code>o</code>.
   * @param o the factor modulator. */
  public void setAbsMod(AObject o) {
    if(o != null) absMod = o.copy(); }

  /** Returns the object to be absoluted. */
  public AObject getObj() { return obj; }

  /** Sets the object to be absoluted to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj(AObject o) { obj = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    double v = obj.nextValue(ch, rate);
    double a = nextObjValue(absMod, ch, rate, abs);
    if(v < 0) v*= -a; return v; }

  public AObject copy() { return new AAbs(this); }
}