// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class adding the values of two {@link ALab.AObject}s.
 * If the addition is out of range (<code>-1..1</code>)
 * an exception is thrown. An alternative possibility to
 * do addition is to use {@link ALab.AAmp AAmp}, that
 * provides faster offset calculation and can add values
 * out of the usual range.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.AAmp AAmp
 */
public class AAdd extends AObject {
  private AObject obj1;
  private AObject obj2;

  /** Creates a new object-adding instance.
   * @param obj1 the first object to be added.
   * @param obj2 the second object to be added. */
  public AAdd(AObject obj1, AObject obj2) {
    super(Math.max(obj1.getChannel(),obj2.getChannel()));
    setObj1(obj1); setObj2(obj2); }

  /** Creates a new <code>AAdd</code>
   * by copying the given <code>AAdd</code>.
   * @param o the given object. */
  protected AAdd(AAdd o) { super(o);
    setObj1(o.getObj1()); setObj2(o.getObj2()); }

  /** Returns the first object that is added. */
  public AObject getObj1() { return obj1; }

  /** Sets the first object that is added
   * to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj1(AObject o) { obj1 = o.copy(); }

  /** Returns the second object that is added. */
  public AObject getObj2() { return obj2; }

  /** Sets the second object that is added
   * to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj2(AObject o) { obj2 = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    return obj1.nextValue(ch, rate) + obj2.nextValue(ch, rate); }

  public AObject copy() { return new AAdd(this); }
}
