// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class multiplying the values of an {@link ALab.AObject} by
 * an amplification value and adding an offset value to the
 * result. <br><br><IMG SRC="space.gif" WIDTH=50>
 * <IMG SRC="AAmp.gif" ALT="Diagram of amplification."><br>
 * Both values - amp and offset - can be modulated.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 */
public class AAmp extends AObject {
  private double  amp;
  private double  offset;
  private AObject ampMod = null;
  private AObject offsetMod = null;
  private AObject obj;

  /** Creates a new amplification instance.
   * @param o the object to be amplified.
   * @param amp the amplification value.
   * @param v the offset value. */
  public AAmp(AObject o, double amp, double v) {
    super(o); setObj(o); setAmp(amp); setOffset(v); }

  /** Creates a new <code>AAmp</code>
   * by copying the given <code>AAmp</code>.
   * @param o the given object. */
  protected AAmp(AAmp o) {    super(o); setObj(o.getObj());
    setAmp(o.getAmp());       setAmpMod(o.getAmpMod());
    setOffset(o.getOffset()); setOffsetMod(o.getOffsetMod()); }

  /** Returns the amplification value. */
  public double getAmp() { return amp; }

  /** Sets the amplification value.
   * @param v the amp value. */
  public void setAmp(double v) { amp = v; }

  /** Returns the amplification modulator. */
  public AObject getAmpMod() { return ampMod; }

  /** Sets the amplification modulator to a copy of <code>o</code>.
   * @param o the amp modulator. */
  public void setAmpMod(AObject o) {
    if(o != null) ampMod = o.copy(); }

  /** Returns the offset value. */
  public double getOffset() { return offset; }

  /** Sets the offset value.
   * @param v the offset value. */
  public void setOffset(double v) { offset = v; }

  /** Returns the offset modulator. */
  public AObject getOffsetMod() { return offsetMod; }

  /** Sets the offset modulator to a copy of <code>o</code>.
   * @param o the offset modulator. */
  public void setOffsetMod(AObject o) {
    if(o != null) offsetMod = o.copy(); }

  /** Returns the object to be amplified. */
  public AObject getObj() { return obj; }

  /** Sets the object to be amplified to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj(AObject o) { obj = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    double a = nextObjValue(   ampMod, ch, rate,    amp);
    double o = nextObjValue(offsetMod, ch, rate, offset);
    return obj.nextValue(ch, rate) * a + o; }

  public AObject copy() { return new AAmp(this); }
}