// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class returning the average of some samples of an {@link
 * ALab.AObject AObject}. Eg. the number of samples that should
 * be computed (the average size) is three. The original values
 * are <code>0.3 0.6 0 -0.9</code>. It is asumed that the values
 * before this sequenze are zero. The average of the three
 * values <code>0 0 0.3</code> is <code>0.1</code>. The average
 * of <code>0 0.3 0.6</code> is <code>0.3</code>... The new
 * returned values are: <code>0.1 0.3 0.3 -0.1</code>.
 * Due to the delay of this algorithm you should move the
 * resulting object the half size forward.<br><br>
 * <IMG SRC="AAvg.gif" ALT="Diagram of the average effect.">
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.AMove AMove
 */
public class AAvg extends AObject {
  private AObject obj;
  private double[][] stack;
  private double[] old;
  private int size;
  private int index = -1;

  /** Creates a new average calculating instance.
   * @param o the average values of this object are calculated.
   * @param size the number of samples to be calculated. */
  public AAvg(AObject o, int size) {
    super(o); setObj(o); setSize(size); }

  /** Creates a new <code>AAvg</code>
   * by copying the given <code>AAvg</code>.
   * @param o the given object. */
  protected AAvg(AAvg o) { super(o);
    setObj(o.getObj()); setSize(o.getSize()); }

  /** Returns the object that will be calculated. */
  public AObject getObj() { return obj; }

  /** Sets the object that will be calculated
   * to a copy of <code>o</code>.
   * @param o the average values of this object are calculated. */
  protected void setObj(AObject o) { obj = o.copy(); }

  /** Returns the number of samples to be calculated. */
  public int getSize() { return size; }

  /** Sets the number of samples to be calculated.
   * A size below one takes no effect.
   * @param size the value of the size. */
  public void setSize(int size) { old = new double[channel];
    if(size < 1) size = 1; this.size = size;
    stack = new double[size][channel];
    for(int i = 0; i < channel; i++) { old[i] = 0;
      for(int j = 0; j < size; j++) stack[j][i] = 0; }}

  private double stack(int ch, double v) {
    if(isNewSample(ch) && (++index >= size)) index = 0;
    double o = stack[index][ch]; stack[index][ch] = v;
    return o; }

  protected double getValue(int ch, int rate) throws AException {
    double v = obj.nextValue(ch, rate);
    double s = stack(ch, v);
    v = old[ch] + (v - s) / size;
    old[ch] = v; return v; }

  public AObject copy() { return new AAvg(this); }
}