// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

import java.io.*;

/**
 * A class buffering values of an {@link ALab.AObject AObject} for
 * faster use. All copies of this ABuffer use the same buffer, so
 * the values are only calculated once. The buffer has a certain
 * length, which tells how many samples are calculated before use.
 * If any value outside the buffer is asked for, zero is returned.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.AStore AStore
 */
public class ABuffer extends AObject {
  private double[][] buf = null;
  private int len;
  private int pos = -1;

  /** Creates a new buffer for faster use of values.
   * @param o the object with the values to be buffered.
   * @param length the amount of samples to be stored.
   * @param rate the samplerate of the calculated values.
   * @exception AException if any calculation error occurs. */
  public ABuffer(AObject o, int length, int rate) throws AException {
    super(o);
    System.out.print("buffering: ->");
    len = Math.max(1, length);
    buf = new double[len][channel];
    for(int i = 0; i < len; i++) {
      if((i % 10000) == 0) System.out.print("\b->");
      for(int ch = 0; ch < channel; ch++)
        buf[i][ch] = o.nextValue(ch, rate); }
    System.out.println("\b- done."); }

  /** Creates a new <code>ABuffer</code> by copying the given
   * <code>ABuffer</code> and using the buffer of the given object.
   * @param o the given object. */
  protected ABuffer(ABuffer o) { super(o); setBuffer(o.getBuffer()); }

  /** Returns the buffer. The first index of the buffer is the
   * sample position the second index is the channel number.
   * @return the buffer containing the values. */
  public double[][] getBuffer() { return buf; }

  /** Sets the pointer of the buffer to the given buffer.
   * @param v the buffer. */
  protected void setBuffer(double[][] v) { buf = v; len = v.length; }

  protected double getValue(int ch, int rate) throws AException {
    if(isNewSample(ch)) pos++;
    if(pos < len) return buf[pos][ch]; else return 0; }

  public AObject copy() { return new ABuffer(this); }
}