// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class joining two {@link ALab.AObject AObject}s.
 * The new object has as much channels as these two
 * objects have together.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.ASplit ASplit
 */
public class ACombine extends AObject {
  private AObject obj1;
  private AObject obj2;
  private int nr;

  /** Creates a new object joining instance.
   * @param obj1 the first object to be joined.
   * @param obj2 the second object to be joined. */
  public ACombine(AObject obj1, AObject obj2) {
    super(obj1.getChannel() + obj2.getChannel());
    setObj1(obj1); setObj2(obj2); }

  /** Creates a new <code>ACombine</code>
   * by copying the given <code>ACombine</code>.
   * @param o the given object. */
  protected ACombine(ACombine o) { super(o);
    setObj1(o.getObj1()); setObj2(o.getObj2()); }

  /** Returns the first object to be joined. */
  public AObject getObj1() { return obj1; }

  /** Sets the first object to be joined
   * to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj1(AObject o) {
    obj1 = o.copy(); nr = obj1.getChannel(); }

  /** Returns the second object to be joined. */
  public AObject getObj2() { return obj2; }

  /** Sets the second object to be joined
   * to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj2(AObject o) { obj2 = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    if(ch < nr) return obj1.nextValue(ch, rate);
    else return obj2.nextValue(ch - nr, rate); }

  public AObject copy() { return new ACombine(this); }
}
