// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class cutting values of {@link ALab.AObject}s. If the
 * cutting limit is positive than all values that are
 * above this limit are truncated to this limit. If the
 * limit is negative than only values above this limit are
 * played, all others are truncated to zero.<br><br>
 * <IMG SRC="ACut.gif" ALT="Diagram of the limit effect.">
 * <br>The limit can be modulated.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 */
public class ACut extends AObject {
  private double  limit;
  private AObject limitMod = null;
  private AObject obj;

  /** Creates a new value cutting instance.
   * @param o the object to be cutted.
   * @param limit the truncating level. */
  public ACut(AObject o, double limit) {
    super(o); setObj(o); setLimit(limit); }

  /** Creates a new <code>ACut</code>
   * by copying the given <code>ACut</code>.
   * @param o the given object. */
  protected ACut(ACut o) {  super(o);
    setLimitMod(o.getLimitMod());
    setLimit(o.getLimit()); setObj(o.getObj()); }

  /** Returns the limit value. */
  public double getLimit() { return limit; }

  /** Sets the limit vlaue.
   * @param v the truncating level. */
  public void setLimit(double v) { limit = v; }

  /** Returns the limit modulator. */
  public AObject getLimitMod() { return limitMod; }

  /** Sets the limit modulator to a copy of <code>o</code>.
   * @param o the limit modulator. */
  public void setLimitMod(AObject o) {
    if(o != null) limitMod = o.copy(); }

  /** Returns the object to be cutted. */
  public AObject getObj() { return obj; }

  /** Sets the object to be cutted to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj(AObject o) { obj = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    double v = obj.nextValue(ch, rate);
    double l = nextObjValue(limitMod, ch, rate, limit) % 1.0;
    if(l >= 0) { if(v > l) v = l; else if(v < -l) v = -l; }
    else { l++; if((v < l) && (v > -l)) v = 0; }
    return v; }

  public AObject copy() { return new ACut(this); }
}