// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * An elliptic waveform generator. <br><br>
 * <IMG SRC="space.gif" WIDTH=50 HEIGHT=1>
 * <IMG SRC="AEll.gif" ALT="Diagram of the waveform.">
 * <br><br><IMG SRC="point.gif">
 * @author  Michael Bliem
 * @version 1.0
 */
public class AEll extends AWave {
  private double factor = 2.0 / Math.PI;

  /**
   * Creates an elliptic waveform generator.
   * @param ch    the amount of audio channels.
   * @param freq  the frequency.
   * @param phase the phase.
   * @see ALab.AWave AWave
   */
  public AEll(int ch, double freq, double phase) {
    super(ch, freq, phase); }

  /** Creates a new <code>AEll</code>
   * by copying the given <code>AEll</code>.
   * @param w the given object. */
  protected AEll(AWave w) { super(w); }

  protected double getValue(int ch, int rate) throws AException {
    double p = nextPhase(ch, rate) * factor;
    if(p < 0) { p+= 1; return -Math.sqrt(1 - p * p); }
    else      { p-= 1; return  Math.sqrt(1 - p * p); }}

  public AObject copy() { return new AEll(this); }
}