// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

import java.io.*;

/**
 * An implementation of {@link ALab.AInputStream AInputStream} using a File.
 * The represented data is read by a {@link java.io.FileInputStream
 * FileInputStream}, and buffered by a {@link java.io.BufferedInputStream
 * BufferedInputStream}.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.AInputStream
 */
public class AFile extends BufferedInputStream implements AInputStream {
  private String name;
  private long pos = 0;

  /**
   * Creates a new buffered input stream that reads data
   * from the file named <code>name</code>.
   * @param name the filename.
   * @exception IOException if an IOError occurs.
   */
  public AFile(String name) throws IOException { this(name, 0); }

  /**
   * Creates a new buffered input stream that reads data from the file
   * named <code>name</code> and skips <code>pos</code> bytes.
   * @param name the filename.
   * @param pos the position of the first byte to read.
   * @exception IOException if an IOError occurs.
   */
  public AFile(String name, long pos) throws IOException {
    super(new FileInputStream(name)); this.name = name;
    mark(Integer.MAX_VALUE); setPos(pos); }

  public long getPos() { return pos; }

  public void setPos(long v) throws IOException {
    v = Math.max(0, v); pos = v; reset();
    while(v > 0) v -= skip(v);
    mark(Integer.MAX_VALUE); }

  public AInputStream copy() throws IOException {
    return new AFile(name, pos); }
}