// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class making any {@link ALab.AObject AObject} to hold its
 * last value until it is triggered. Then it holds a new value
 * until it is triggered again (sample and hold algorithm).
 * An AHold object is triggered by any AObject. If the value of
 * the AObject is above zero, it is interpreted as a trigger
 * impuls. There are some classes that are constructed to work as
 * triggers. These trigger objects return <code>1</code> if they
 * trigger and <code>0</code> if they don't.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.ASimpleTrigger ASimpleTrigger
 * @see     ALab.ARandomTrigger ARandomTrigger
 * @see     ALab.AWaveTrigger AWaveTrigger
 * @see     ALab.AZeroTrigger AZeroTrigger
 */
public class AHold extends AObject {
  private AObject  obj;
  private AObject  trigger;
  private double[] hold;

  /**
   * Creates a new sample and hold object. The values of
   * <code>o</code> are only refreshed when the value of
   * <code>trigger</code> are above zero.
   * @param o the object holding the values.
   * @param trigger the trigger object.
   */
  public AHold(AObject o, AObject trigger) {
    super(o); setObj(o); setTrigger(trigger); }

  /** Creates a new <code>AHold</code>
   * by copying the given <code>AHold</code>.
   * @param o the given object. */
  protected AHold(AHold o) { super(o);
    setObj(o.getObj()); setTrigger(o.getTrigger()); }

  /** Returns the object holding the values. */
  public AObject getObj() { return obj; }

  /** Sets the object holding the values to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj(AObject o) { obj = o.copy(); }

  /** Returns the trigger object. */
  public AObject getTrigger() { return trigger; }

  /** Sets the trigger object to a copy of <code>t</code>.
   * @param t the trigger object. */
  protected void setTrigger(AObject t) {
    trigger = t.copy(); hold = arrayWith(0); }

  protected double getValue(int ch, int rate) throws AException {
    double v = obj.nextValue(ch, rate);
    if(trigger.nextValue(ch, rate) > 0) hold[ch] = v;
    return hold[ch]; }

  public AObject copy() { return new AHold(this); }
}
