// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

import java.io.*;

/**
 * This interface describes nearly the class {@link java.io.InputStream
 * InputStream}, but can copy itself and get and set the marked position.
 * The method {@link #copy() copy()} returns another InputStream representing
 * the same data. This method is needed to comletely copy {@link ALab.AObject
 * AObject}s that use InputStreams (eg. {@link ALab.ASample ASample}),
 * because usual InputStreams cannot be copied. The copied input stream must
 * be set to the same position as the original one.<br><br> To ensure full
 * compatibility to {@link java.io.InputStream InputStream}, classes
 * implementing this interface should be subclasses of <code>InputStream</code>.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     java.io.InputStream java.io.InputStream
 * @see     ALab.AFile AFile
 */
public interface AInputStream {
  public int     available()                      throws IOException;
  public void    close()                          throws IOException;
  public void    mark(int readlimit);
  public boolean markSupported();
  public int     read()                           throws IOException;
  public int     read(byte[] b)                   throws IOException;
  public int     read(byte[] b, int off, int len) throws IOException;
  public void    reset()                          throws IOException;
  public long    skip(long n)                     throws IOException;
  /** Returns marked position. */
  public long getPos();
  /** Sets position to be marked.
   * @param v the amount of bytes to skip. */
  public void setPos(long v)                   throws IOException;
  /** Returns another AInputStream representing the same data. If the
   * current position is marked, then the copy has to be marked, too.
   * @return    a copy of itself.
   * @exception IOException if an IOError occurs. */
  public AInputStream copy()                           throws IOException; }