// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class moving an {@link ALab.AObject AObject} in time. If the
 * object is moved backwards it will start playing later, if it
 * is moved forward some samples will be skipped (not be played).
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 */
public class AMove extends AObject {
  private long length;
  private long pos = 0;
  private AObject obj;

  /** Creates a new AObject moving object.
   * @param o the AObject to be moved.
   * @param length the amount of samples to be moved. */
  public AMove(AObject o, long length) {
    super(o); setObj(o); setLength(length); }

  /** Creates a new <code>AMove</code>
   * by copying the given <code>AMove</code>.
   * @param o the given object. */
  protected AMove(AMove o) { super(o);
    setObj(o.getObj()); setLength(o.getLength()); }

  /** Returns the amount of samples that will be moved. */
  public long getLength() { return length; }

  /** Sets the amount of samples that will me moved. If length
   * is positive, length zero samples will be played before the
   * AObjects values will be played. If length is negative,
   * length samples of AObject will be skipped.
   * @param v the length. */
  public void setLength(long v) { if(v < 0) pos = v; length = v; }

  /** Returns the object that will be moved. */
  public AObject getObj() { return obj; }

  /** Sets the object that will be moved to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj(AObject o) { obj = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    if(isNewSample(ch)) while(pos++ < 0)
      for(int i = 0; i < channel; i++) obj.nextValue(i, rate);
    if(pos < length) return 0;
    else return obj.nextValue(ch, rate); }

  public AObject copy() { return new AMove(this); }
}