// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class multiplying the values of two {@link ALab.AObject}s.
 * An alternative possibility to do multiplication is to use
 * {@link ALab.AAmp AAmp}, that provides faster amplification
 * calculation and can multiply values out of the usual range.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.AAmp AAmp
 */
public class AMul extends AObject {
  private AObject obj1;
  private AObject obj2;

  /** Creates a new object-multiplying instance.
   * @param obj1 the first object to be multiplied.
   * @param obj2 the second object to be multiplied. */
  public AMul(AObject obj1, AObject obj2) {
    super(Math.max(obj1.getChannel(),obj2.getChannel()));
    setObj1(obj1); setObj2(obj2); }

  /** Creates a new <code>AMul</code>
   * by copying the given <code>AMul</code>.
   * @param o the given object. */
  protected AMul(AMul o) { super(o);
    setObj1(o.getObj1()); setObj2(o.getObj2()); }

  /** Returns the first object that is multiplied. */
  public AObject getObj1() { return obj1; }

  /** Sets the first object that is multiplied
   * to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj1(AObject o) { obj1 = o.copy(); }

  /** Returns the second object that is multiplied. */
  public AObject getObj2() { return obj2; }

  /** Sets the second object that is multiplied
   * to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj2(AObject o) { obj2 = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    return obj1.nextValue(ch, rate) * obj2.nextValue(ch, rate); }

  public AObject copy() { return new AMul(this); }
}
