// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * Makes any {@link ALab.AObject AObject} to have a certain length.
 * After this length, <code>nextValue()</code> returns zero and the
 * method {@link #endReached() endReached()} returns <code>true</code>.
 * Minimum amount of length is zero. Length of one sample is not
 * possible and interpreted as two. <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 */
public class APart extends AObject {
  private AObject obj;
  private long length;
  private long pos = 0;
  private int  endChannel;
  private boolean end = false;

  /** Creates a new part of an AObject.
   * @param o the AObject.
   * @param length the number of samples this part plays. */
  public APart(AObject o, long length) {
    super(o); setObj(o); setLength(length); }

  /** Creates a new <code>APart</code>
   * by copying the given <code>APart</code>.
   * @param o the given object. */
  protected APart(APart o) {  super(o);
    setLength(o.getLength()); setObj(o.getObj()); }

  /** Returns the length of this part. */
  public long getLength() { return length; }

  /** Sets the length of this part. <code>v</code> samples
   * are calculated before this part returns zero. Minimum
   * length is zero, one is not possible (interpreted as two).
   * @param v number of samples to be played. */
  public void setLength(long v) {
    if(v <= 0) { end = true; length = 0; }
    else length = Math.max(2, v); }

  /** Returns the object that has been lengthened. */
  public AObject getObj() { return obj; }

  /** Sets the object that will be lengthened
   * to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj(AObject o) { obj = o.copy(); }

  /** Marks weather end has been reached or not.
   * @return <code>true</code>: if end has been reached.
   *         <code>false</code>: if end not reached jet. */
  public boolean endReached() { return end; }

  protected double getValue(int ch, int rate) throws AException {
    if(isNewSample(ch)) { pos++; endChannel = last; }
    if(end) return 0;
    if((ch == endChannel) && (pos == length)) end = true;
    return obj.nextValue(ch, rate); }

  public AObject copy() { return new APart(this); }
}