// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class representing a ramp. A ramp has a start value, an end
 * value and a length to reach the end value. The first value is
 * the start value, after length samples the endvalue is reached.
 * Both - start and end value - can be modulated. After length
 * samples there is always returned the end value.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 */
public class ARamp extends    AObject {
  private double  begin;
  private double  end;
  private AObject beginMod;
  private AObject endMod;
  private long    length;
  private long    pos = -1;

  /** Creates a new ramp.
   * @param ch the amount of channels.
   * @param begin the start value.
   * @param end the end value.
   * @param length the amount of samples to reach end.
   */
  public ARamp(int ch, double begin, double end, long length) {
    super(ch); setBegin(begin); setEnd(end); setLength(length); }

  /** Creates a new <code>ARamp</code>
   * by copying the given <code>ARamp</code>.
   * @param o the given object. */
  protected ARamp(ARamp o) {  super(o);
    setBegin(o.getBegin());   setBeginMod(o.getBeginMod());
    setEnd(o.getEnd());       setEndMod(o.getEndMod());
    setLength(o.getLength()); }

  /** Returns the amount of samples to reach the end value. */
  public long getLength() { return length; }

  /** Sets the amount of samples to reach the end value.
   * @param v the length. */
  public void setLength(long v) { length = v; }

  /** Returns the start value. */
  public double getBegin() { return begin; }

  /** Sets the start value.
   * @param v the start value. */
  public void setBegin(double v) { begin = v; }

  /** Returns the start value modulator. */
  public AObject getBeginMod() { return beginMod; }

  /** Sets the start value modulator to a copy of <code>o</code>.
   * @param o the modulator. */
  public void setBeginMod(AObject o) {
    if(o != null) beginMod = o.copy(); }

  /** Returns the end value. */
  public double getEnd() { return end; }

  /** Sets the end value.
   * @param v the end value. */
  public void setEnd(double v) { end = v; }

  /** Returns the end value modulator. */
  public AObject getEndMod() { return endMod; }

  /** Sets the end value modulator to a copy of <code>o</code>.
   * @param o the modulator. */
  public void setEndMod(AObject o) {
    if(o != null) endMod = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    double b = nextObjValue(beginMod, ch, rate, begin);
    double e = nextObjValue(  endMod, ch, rate,   end);
    if(isNewSample(ch)) pos++;
    if(pos >= length) return end;
    else return b + (e - b) * ((double)pos/length); }

  public AObject copy() { return new ARamp(this); }
}
