// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

import java.util.*;

/**
 * A class triggering randomly. To control the amount of triggers
 * per time, the width defines that about every width samples a
 * trigger occurs. This width can be modulated.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.AHold AHold
 */
public class ARandomTrigger extends AObject {
  private Random  rand = new Random();
  private double  width;
  private AObject widthMod = null;

  /** Creates a new randomly triggering object.
   * @param ch the amount of channels.
   * @param width triggers about every <code>width</code>
   * samples. */
  public ARandomTrigger(int ch, double width) {
    super(ch); setWidth(width); }

  /** Creates a new <code>ARandomTrigger</code>
   * by copying the given <code>ARandomTrigger</code>.
   * @param o the given object. */
  protected ARandomTrigger(ARandomTrigger o) { super(o);
    setWidth(o.getWidth()); setWidthMod(o.getWidthMod()); }

  /** Returns the width of the trigger. */
  public double getWidth() { return width; }

  /** Sets the width of the trigger. About every
   * <code>width</code> sample a trigger occurs.
   * @param v the width. */
  public void setWidth(double v) { width = v; }

  /** Returns the width modulator. */
  public AObject getWidthMod() { return widthMod; }

  /** Sets the width modulator to a copy of <code>o</code>.
   * @param o the width modulator. */
  public void setWidthMod(AObject o) {
    if(o != null) widthMod = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    double w = nextObjValue(widthMod, ch, rate, width);
    if(rand.nextInt((int)Math.max(Math.abs(w), 1)) == 0)
      return 1; else return 0; }

  public AObject copy() { return new ARandomTrigger(this); }
}