// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A linear waveform generator. This waveform is used
 * for wavetable synthesis. The values run from <code>0</code>
 * to <code>1</code> if the phase is below <code>PI</code> and
 * from <code>0</code> to <code>-1</code> if the phase is above
 * <code>PI</code>. A new waveform can simply be generated by
 * using this waveform together with {@link ALab.ATablelize
 * ATablelize}.<br><br>
 * <IMG SRC="space.gif" WIDTH=50 HEIGHT=1>
 * <IMG SRC="ASimpleWave.gif" ALT="Diagram of the waveform.">
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see ALab.ATablelize ATablelize
 */
public class ASimpleWave extends AWave {
  /**
   * Creates a linear waveform generator.
   * @param ch    the amount of audio channels.
   * @param freq  the frequency.
   * @param phase the phase.
   * @see ALab.AWave AWave
   */
  public ASimpleWave(int ch, double freq, double phase) {
    super(ch, freq, phase); }

  /** Creates a new linear by copying the given wave object.
   * @param w the given object. */
  protected ASimpleWave(AWave w) { super(w); }

  protected double getValue(int ch, int rate) throws AException {
    double p = nextPhase(ch, rate);
    if(p >= 0) return p / Math.PI; else return -p / Math.PI - 1; }

  public AObject copy() { return new ASimpleWave(this); }
}