// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class representing one audio channel of an {@link
 * ALab.AObject AObject}. The splitchannel tells which
 * channel of the object is represented. The new amount
 * of channels is one and the number is zero, but the
 * represented values are those of the splitchannel of
 * the object that is splitted.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.ACombine ACombine
 */
public class ASplit extends AObject {
  private int splitChannel;
  private AObject obj;

  /** Creates a new object splitting instance. Only one
   * audio channel will be represented.
   * @param o the object to be splitted.
   * @param ch the splitchannel. */
  public ASplit(AObject o, int ch) {
    super(1); setObj(o); setSplitChannel(ch); }

  /** Creates a new <code>ASplit</code>
   * by copying the given <code>ASplit</code>.
   * @param o the given object. */
  protected ASplit(ASplit o) { super(o);
    setObj(o.getObj()); setSplitChannel(o.getSplitChannel()); }

  /** Returns the splitchannel. */
  public int getSplitChannel() { return splitChannel; }

  /** Sets the splitchannel.
   * @param ch the channel that will be represented. */
  public void setSplitChannel(int ch) {
    if(ch >= obj.getChannel()) ch = obj.getChannel() - 1;
    splitChannel = Math.max(ch, 0); }

  /** Returns the object to be splitted. */
  public AObject getObj() { return obj; }

  /** Sets the object to be splitted to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj(AObject o) { obj = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    return obj.nextValue(splitChannel, rate); }

  public AObject copy() { return new ASplit(this); }
}