// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A square generator. Usually a square waveforms value is
 * <code>1</code> for phases below <code>PI</code> and <code>-1</code>
 * for values above <code>PI</code> (50% of the values are high state,
 * 50% are low state). This ratio can be changed by the parameter
 * <code>width</code> and modulated by <code>widthMod</code>.<br><br>
 * <IMG SRC="ASquare.gif" ALT="Diagram of the waveform.">
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 */
public class ASquare extends AWave {
  private double  width;
  private AObject widthMod = null;

  /**
   * Creates a square generator.
   * @param ch    the amount of audio channels.
   * @param freq  the frequency.
   * @param phase the phase.
   * @param width the ratio of high state and low state.
   * @see ALab.AWave AWave
   */
  public ASquare(int ch, double freq, double phase, double width) {
    super(ch, freq, phase); setWidth(width); }

  /** Creates a new <code>ASquare</code>
   * by copying the given <code>ASquare</code>.
   * @param w the given object. */
  protected ASquare(ASquare w) { super(w);
    setWidth(w.getWidth()); setWidthMod(w.getWidthMod()); }

  /** Returns the width of the waveform. */
  public double getWidth() { return width; }

  /** Sets the width of the waveform. If <code>v</code> is a
   * value near zero, the ratio of low state and high state is
   * near 1, if the value
   * is higher, the waveform becomes sharp.
   * @param v the flattnes or sharpness of the waveform. */
  public void setWidth(double v) { width = v; }

  /** Returns the width modulator. */
  public AObject getWidthMod() { return widthMod; }

  /** Sets the width modulator to a copy of <code>o</code>.
   * @param o the width modulator. */
  public void    setWidthMod(AObject o) {
    if(o != null) widthMod = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    double m = (nextObjValue(widthMod, ch, rate, width) / Math.PI) % 4.0;
    double p = nextPhase(ch, rate) / Math.PI;
    if(p < 0) p+= 2;
    if(m < -1) m+= 4; else if(m >= 3) m-= 4;
    if(m < 1) if(p > (m + 1)) return -1; else return 1;
    else      if(p < (m - 1)) return -1; else return 1; }

  public AObject copy() { return new ASquare(this); }
}