// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class triggering every new circle of an {@link ALab.AWave AWave}.
 * Every time the waves method {@link ALab.AWave#isZero(int) isZero()}
 * returns <code>true</code>, a trigger is thrown.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.AWave AWave
 * @see     ALab.AHold AHold
 */
public class AWaveTrigger extends AObject {
  private AWave obj;

  /** Creates a new circle triggering object.
   * @param o the wave object that is asked for new circles. */
  public AWaveTrigger(AWave o) { super(o); setObj(o); }

  /** Creates a new <code>AWaveTrigger</code>
   * by copying the given <code>AWaveTrigger</code>.
   * @param o the given object. */
  protected AWaveTrigger(AWaveTrigger o) { super(o); setObj(o.getObj()); }

  /** Returns the wave object that is called for new circles. */
  public AWave getObj() { return obj; }

  /** Sets the wave object that is called for new circles
   * to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj(AWave o) { obj = (AWave)o.copy(); }

  protected double getValue(int ch, int rate) throws AException {
    double v = obj.nextValue(ch, rate);
    if(obj.isZero(ch)) return 1; else return 0; }

  public AObject copy() { return new AWaveTrigger(this); }
}