// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

package ALab;

/**
 * A class triggering every time an {@link ALab.AObject AObject}s
 * values are crossing zero. If the last value was above (below)
 * zero and this value is below (above) zero, a trigger is thrown.
 * <br><br><IMG SRC="point.gif">
 *
 * @author  Michael Bliem
 * @version 1.0
 * @see     ALab.AHold
 */
public class AZeroTrigger extends AObject {
  private AObject  obj;
  private double[] old;

  /** Creates a new zerocrossing triggering object.
   * @param o the object asked for crossing zero. */
  public AZeroTrigger(AObject o) { super(o); setObj(o); }

  /** Creates a new <code>AZeroTrigger</code>
   * by copying the given <code>AZeroTrigger</code>.
   * @param o the given object. */
  protected AZeroTrigger(AZeroTrigger o) {
    super(o); setObj(o.getObj()); }

  /** Returns the object asked for crossing zero. */
  public AObject getObj() { return obj; }

  /** Sets the object asked for crossing zero
   * to a copy of <code>o</code>.
   * @param o the object. */
  protected void setObj(AObject o) {
    obj = o.copy(); old = arrayWith(0); }

  protected double getValue(int ch, int rate) throws AException {
    double r;
    double v = obj.nextValue(ch, rate);
    if(((v == 0) && (old[ch] != 0))
    || ((v  < 0) && (old[ch]  > 0))
    || ((v  > 0) && (old[ch]  < 0))) r = 1; else r = 0;
    old[ch] = v; return r; }

  public AObject copy() { return new AZeroTrigger(this); }
}