// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

import ALab.*;

public class AExample {
  public static AObject ASinFM(int ch, double hz1, double hz2, double amount) {
    ASin s = new ASin(ch, hz1*2, Math.PI/2);
    s.setFreqMod(new AAmp(new ASin(ch, hz2, 0), 0.5*amount, 0.5));
    return s; }

  public static AObject AEnv(AObject obj, long a, long d, double s, long l, long r) {
    return AEnv(obj, 0, a, d, s, l, r); }

  public static AObject AEnv(AObject obj, long w, long a, long d, double s, long l, long r) {
    int ch = obj.getChannel();
    AContainer c = new AContainer(ch);
    c.append(new ARamp(ch, 0, 0, w), w);
    c.append(new ARamp(ch, 0, 1, a), a);
    c.append(new ARamp(ch, 1, s, d), d+l);
    c.append(new ARamp(ch, s, 0, r), r);
    return new AMul(obj, c); }

  public static AObject AMix(AObject obj1, double amp1, AObject obj2, double amp2) {
    if(amp1 != 1) obj1 = new AAmp(obj1, amp1, 0);
    if(amp2 != 1) obj2 = new AAmp(obj2, amp2, 0);
    return new AAdd(obj1, obj2); }

  public static AObject ADelay(AObject obj, long time, int echos) {
    AObject mix = new AAvg(obj, 1<<echos);
    for(int i = 1; i < echos; i++)
      mix = AMix(new AAvg(obj, 1<<(echos-i)), 1, new AMove(mix, time), 0.5);
    return AMix(obj, 1, new AMove(mix, time), 0.5); }

  public static AObject AReverb(AObject obj, int delays, long time, int echos) {
    AObject mix = new AAmp(ADelay(obj, time, echos), 1.0/delays, 0);
    for(int i = 2; i <= delays; i++)
      mix = AMix(mix, 1, ADelay(obj, time + time/i, echos), 1.0/delays);
    return mix; }

  public static AObject AMono(AObject obj, double amp) {
    AObject mix = new ASplit(obj, 0);
    if(amp != 1) mix = new AAmp(mix, amp, 0);
    for(int i = 1; i < obj.getChannel(); i++)
      mix = AMix(mix, 1, new ASplit(obj, i), amp);
    return mix; }

  public static AObject AGroundFilter(AObject obj, int size) {
    return new ASub(new AMove(obj, size/2), new AAvg(obj, size)); }
}