// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

import ALab.*;

/** An impuls generator.
 * @author  Michael Bliem
 * @version 1.0 */
public class AImpuls extends AWave {

  // stores the last sign at each channel
  // true for positive phase
  private boolean[] oldSign;

  /** Creates an impuls generator.
   * @param ch    the amount of audio channels.
   * @param freq  the frequency.
   * @param phase the phase. */
  public AImpuls(int ch, double freq, double phase) {

    // calls the constructor of AWave
    super(ch, freq, phase);

    // inits the oldSign array
    setArray(); }

  /** Creates a new <code>AImpuls</code>
   * by copying the given <code>AImpuls</code>.
   * @param w the given object. */
  protected AImpuls(AWave w) {

    // calls the copying constructor of AWave
    super(w);

    // inits the oldSign array
    setArray(); }

  // oldSign initialized depending on phase of this wave
  private void setArray() {
    oldSign = new boolean[channel];
    for(int i = 0; i < channel; i++)
      oldSign[i] = (getPhase() >= 0); }

  protected double getValue(int ch, int rate) throws AException {
    double result;

    // current sign depends on current phase
    // (nextPhase() is called! must be called before isZero())
    boolean currentSign = (nextPhase(ch, rate) >= 0);

    // if new circle starts return 1
    if(isZero(ch)) result = 1.0;

    // if old sign is different from current sign return -1
    else if(oldSign[ch] != currentSign) result = -1.0;

    // in all other cases return 0
    else result = 0.0;

    // current sign is stored as old sign
    oldSign[ch] = currentSign;

    return result; }

  public AObject copy() { return new AImpuls(this); }
}

