// ALab v1.0: java package for creating experimental digital sounds and soundeffects
// Copyright (C) 2001 Michael Bliem   ALab@gmx.at      see 'license.txt' for details

import ALab.*;

public class AUseMod extends AObject {
  private double  pan; // range: -1(left)..0(middle)..1(right)
  private AObject panMod = null; // modulator
  private AObject obj;

  public AUseMod(AObject o, double pan) {
    super(o.getChannel());
    setObj(o); setPan(pan); }
    // modulator can but must not be set
    // so the modulator is not part of constructor

  protected AUseMod(AUseMod o) { super(o);
    setObj(o.getObj()); setPan(o.getPan());

    // the copying constructor sets the modulator
    // to the modulator of the copied object or to
    // null if copied object has no modulator
    setPanMod(o.getPanMod()); }

  public double getPan() { return pan; }
  public void setPan(double v) {
    if     (v < -1) v = -1;
    else if(v >  1) v =  1; // limited to range -1..1
    pan = v; }

  public AObject getPanMod() { return panMod; }
  public void setPanMod(AObject o) {

    // modulator set to a copy of o if o is not null
    // if o is null modulator stays null
    if(o != null) panMod = o.copy(); }

  public AObject getObj() { return obj; }
  protected void setObj(AObject o) { obj = o.copy(); }

  protected double getValue(int ch, int rate) throws AException {

    // modulated pan value (modPan) is calculated like this:
    // if panMod does not exist (is null) pan is returned
    // else pan is multiplied with next panMod value at channel ch
    // (modPan not used in this example)
    double modPan = nextObjValue(panMod, ch, rate, pan);

    return obj.nextValue(ch, rate); }

  public AObject copy() { return new AUseMod(this); }
}

